// **********************************************************************
//
// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

#include <Ice/Ice.h>
#include <IceUtil/IceUtil.h>

#include <Printer.h>

using namespace std;
using namespace Demo;

class MyThread: public ::IceUtil::Thread
{
public:
    
	MyThread(Ice::ConnectionPtr conn)
	{
		_conn = conn;
	}

	virtual void run()
	{
		sleep(5);
		cout << "client begin close the connection" << endl;
		_conn->close(false);
		cout << "client end of close the connection" << endl;
	}
private:
	Ice::ConnectionPtr _conn;
};

typedef ::IceUtil::Handle< MyThread > MyThreadPtr;

	
int
main(int argc, char * argv[])
{
    int status = 0;
    Ice::CommunicatorPtr ic;
    try {
	ic = Ice::initialize(argc, argv);
	Ice::ObjectPrx base = ic->stringToProxy(
				"SimplePrinter:default -p 10000");
	PrinterPrx printer = PrinterPrx::checkedCast(base);
	if (!printer) throw "Invalid proxy";

	//get the connection
	Ice::ConnectionPtr conn = printer->ice_connection();

	//start the thread
	MyThreadPtr myThread = new MyThread(conn);
	::IceUtil::ThreadControl control = myThread->start();
	

	for(int i=0; i<2; i++)
	{
		try
		{
			cout << "[" << i << "]" << " Client begin send request" << endl;
			printer->printString(i);
			cout << "[" << i << "]" << " Client end of send request" << endl;
		}
		catch (const Ice::Exception & ex) {
			cerr << ex << endl;
			cout << "[" << i << "]" << " Client catch an Ice::Exception" << endl;
		}
		catch (const char *msg) {
			cerr << msg << endl;
			cout << "[" << i << "]" << " Client catch an ::exception" << endl;
		}
	}

	control.join();

    } catch (const Ice::Exception & ex) {
	cerr << ex << endl;
	status = 1;
    } catch (const char * msg) {
	cerr << msg << endl;
	status = 1;
    }

    if (ic) {
	try {
	    ic->destroy();
	} catch (const Ice::Exception & ex) {
	    cerr << ex << endl;
	    status = 1;
	}
    }
    return status;
}
